/*** LINES3.C - Display file type, number of lines and characters.
 *   Version 3.05.19980804.
 *
 *   (C) Fedon Kadifeli, 1991-1998.
 *
 *   Usage: Better to be called from the following batch file (LINES.BAT):
 *
 *      @ECHO OFF
 *      :START
 *      IF "%1"==""  GOTO EXIT
 *      FOR %%F IN (%1) DO C:\BINB\LINES3 %%F
 *      SHIFT
 *      GOTO START
 *      :EXIT
 */

#include <stdio.h>
#include <string.h>

typedef struct {
  char * typ;
  int    start;
  int    len;
  char * sstr;
} sstruct;

char buf [BUFSIZ];

void pfinfo (char * fn, char * b)
{
  static sstruct const ss[] = {
      /* >= 6 */
    { "LICENSE",    0, 9, "Copyright" },
    { "VOC_CREATV", 0, 8, "Creative" },
    { "AIX_ARCH.",  0, 7, "<aiaff>" },
    { "REGEDIT",    0, 7, "REGEDIT" },
    { "ASCII_CPIO", 0, 6, "070707" },
    { "JAVA_PROG.", 0, 6, "import" },
      /* 5 */
    { "AWK_SCRIPT", 0, 5, "BEGIN" },
    { "EPS",        0, 5, "\xC5\xD0\xD3\xC6\x1E" },
    { "METAFILE",   0, 5, "\xD7\xCD\xC6\x9A\0" },
    { "CODE_PAGE",  0, 5, "\xFF" "FONT" },
    { "JPEG_IMAGE", 0, 5, "\xFF\xD8\xFF\xE0\0" },
    { "TAR_ARCH.",257, 5, "ustar" },
    { "RTF_DOC",    0, 5, "{\\rtf" },
      /* 4 */
    { "HELP_FILE",  0, 4, "?_\x03\0" },
    { "POSTSCRIPT", 0, 4, "%!PS" },
    { "PDF_DOCMNT", 0, 4, "%PDF" },
    { "PDF_INDEX",  0, 4, "%PDX" },
    { "AU_AUDIO",   0, 4, ".snd" },
    { "SV_ARCH.",   0, 4, "<ar>" },
    { "CREG",       0, 4, "CREG" },
    { "TERM",       0, 4, "DCS " },
    { "IFF_AUDIO",  0, 4, "FORM" },
    { "MAIL_FILE",  0, 4, "From" },
    { "TIFF",       0, 4, "II*\0" },
    { "HELP_FILE?", 0, 4, "LN\x02\0" },
    { "AUDIO",      0, 4, "MThd" },
    { "CABINET",    0, 4, "MSCF" },
    { "PKZIP_ARCH", 0, 4, "PK\x03\x04" },
    { "GROUP",      0, 4, "PMCC" },
    { "WAV_AUDIO",  0, 4, "RIFF" },
    { "MPEG_VIDEO", 0, 4, "\0\0\x01\xB3" },
    { "ICON",       0, 4, "\0\0\x01\0" },
    { "TT_FONT",    0, 4, "\0\x01\0\0" },
    { "ACCESS",     0, 4, "\x01\0\0\0" },
    { "BACK/REST",  0, 4, "\x09\0\x6B\xEA" },
    { "ELF",        0, 4, "\x7F" "ELF" },
    { "PWD_LIST",   0, 4, "\xB0" "MFN" },
    { "JAVA_CLASS", 0, 4, "\xCA\xFE\xBA\xBE" },
    { "WORD2",      0, 4, "\xDB\xA5\x2D\0" },
    { "WORD6",      0, 4, "\xD0\xCF\x11\xE0" },
    { "POWERPOINT", 0, 4, "\xED\xDE\xAD\x0B" },
    { "LIBRARY",    0, 4, "\xF0\x0D\0\0" },
    { "SYS",        0, 4, "\xFF\xFF\xFF\xFF" },
      /* 3 */
    { "GIF_IMAGE",  0, 3, "GIF" },
    { "CARD_FILE",  0, 3, "MGC" },
    { "ZOO_ARCH.",  0, 3, "ZOO" },
    { "WRITE",      0, 3, "1\xBE\0" },
    { "WRITE",      0, 3, "2\xBE\0" },
    { "PCX",        0, 3, "\x0A\x05\x01" },
    { "LHARC/LARC", 2, 3, "-lh" },
    { "LHARC/LARC", 2, 3, "-lz" },
      /* 2 */
    { "SCRIPT",     0, 2, "#!" },
    { "SCRIPT",     0, 2, "# " },
    { "C_PROGRAM",  0, 2, "/*" },
    { "C++_PROG.",  0, 2, "//" },
    { "WIN_BITMAP", 0, 2, "BM" },
    { "KEYBOARD",   0, 2, "DS" },
    { "EXECUTABLE", 0, 2, "MZ" },
    { "TRANS",      0, 2, "TN" },
    { "LINK",       0, 2, "L\0" },
    { "PBITMAP_AS", 0, 2, "P1" },
    { "PGRYMAP_AS", 0, 2, "P2" },
    { "PPIXMAP_AS", 0, 2, "P3" },
    { "PBITMAP_RW", 0, 2, "P4" },
    { "PGRYMAP_RW", 0, 2, "P5" },
    { "PPIXMAP_RW", 0, 2, "P6" },
    { "PKZIP_ARCH", 1, 2, "PK" },
    { "CPIO_ARCH.", 0, 2, "\x71\xC7" },
    { "ARC_ARCH.",  0, 2, "\x1A\x08" },
    { "PACKED_TXT", 0, 2, "\x1F\x1E" },
    { "GZIP_ARCH.", 0, 2, "\x1F\x8B" },
    { "COMPR_DATA", 0, 2, "\x1F\x9D" },
    { "MPEG-3",     0, 2, "\xFF\xFB" },
      /* 1 */
    { "C_PROGRAM",  0, 1, "#" },
    { "HTML_DOC.",  0, 1, "<" },
    { "INI_FILE",   0, 1, "[" },
    { "ASSEM_PROG", 0, 1, ";" },
    { "FORTRAN_PR", 0, 1, "C" },
    { "EXCEL",      0, 1, "\x09" },
    { "OBJECT_FIL", 0, 1, "\x80" },
    { "DOS_COM_F.", 0, 1, "\xE9" },
    { "DOS_COM_F.", 0, 1, "\xEB" },
      /* 0 */
    { "--- ?? ---", 0, 0, "" }
  };
  register int i;
  int const ssl = (sizeof ss)/(sizeof (sstruct));

  for (i=0; i<ssl; i++)
    if (memcmp(buf+ss[i].start, ss[i].sstr, ss[i].len) == 0)
      break;
  printf ("%-10s %-32s", ss[i].typ, fn);
} /* pfinfo */


int countlines (char * fn)
{
  register size_t bc, i;
  long   lc = 0L;
  long   cc = 0L;
  int    c  = '\n';
  FILE * fp = fopen (fn, "rb");

  if (fp == NULL) {
    fprintf (stderr, "Cannot open '%s'\n", fn);
    return 1;
  }
  memset (buf, '\0', BUFSIZ);
  bc = fread (buf, 1, BUFSIZ, fp);
  pfinfo (fn, buf);
  do {
    cc += bc;
    for (i=0; i<bc; i++)
      if ((c=buf[i]) == '\n')
        lc++;
  } while ((bc = fread(buf, 1, BUFSIZ, fp)) > 0);
  fclose (fp);
  lc += (c!='\n');      /* if incomplete line */
  printf (" %9ld lines, %9ld chars\n", lc, cc);
  return 0;
} /* countlines */


int main (int argc, char **argv)
{
  int i;

  for (i=1; i<argc; i++)
    countlines (argv[i]);
  return 0;
} /* main */
